@@ -1,30 +1,35 @@
 from __future__ import annotations
+from asyncio.futures import Future
 
 import sys
 from os import environ, execl
 
 from asyncio import (
-    get_event_loop,
-    run_coroutine_threadsafe,
-    iscoroutinefunction,
     wait_for,
-    AbstractEventLoop
+    new_event_loop,
+    AbstractEventLoop,
+    iscoroutinefunction,
+    run_coroutine_threadsafe
 )
 from dotenv import load_dotenv
-from typing import Any, Callable, Awaitable, Coroutine, NoReturn
+from typing import (
+    Any,
+    Callable,
+    Awaitable,
+    Coroutine,
+    NoReturn
+)
 from pathlib import Path
 from threading import Thread
 from subprocess import run
 
 from ujson import loads
 from aiohttp import ClientSession
-from colorama import Fore, init
-init()
+from colorama import Fore, Style, init
 
-from . import obj
 from .ws import Ws
 from .db import DB
-from .obj import Message
+from .obj import Message, clear
 from .exceptions import (
     AccountNotFoundInDotenv,
     EventIsntAsync,
@@ -34,13 +39,28 @@ from .exceptions import (
     InvalidEvent
 )
 
-Coro_return_Message = Callable[[], Coroutine[Message, None, None]]
+Coro_return_chat_message = Callable[[], Coroutine[Message.from_ws, None, None]]
 Coro_return_Any = Callable[[], Coroutine[Any, None, None]]
 Coro_return_None = Callable[[], Coroutine[None, None, None]]
 
-version = '0.0.30'
+version = '0.0.38'
 
 class Bot:
+    __slots__ = (
+        '_db',
+        '_email',
+        '_loop',
+        '_msg',
+        '_password',
+        '_ws',
+        'commands',
+        'events',
+        'id',
+        'ignore_chats',
+        'only_chats',
+        'prefix'
+    )
+
     def __init__(
         self,
         email:        str | None           = None,
@@ -49,9 +69,8 @@ class Bot:
         only_chats:   dict[str, list[str]] = {},
         ignore_chats: dict[str, list[str]] = {}
     ):
-        # load_dotenv is not identifying the .env on project folder,
-        # so I use Path to get the absolute .env path
-        load_dotenv(Path('.env').absolute())
+        init()
+        load_dotenv()
         try:
             self._email    = email    or environ['EMAIL']
             self._password = password or environ['PASSWORD']
@@ -63,34 +82,34 @@ class Bot:
         if only_chats and ignore_chats:
             raise InvalidChatChoice('Enter chats only in "only_chats" or "ignore_chats"')
 
-        obj.clear()
+        clear()
 
-        self.id:      str | None        = None
-        self._db:     DB                = DB()
-        self._msg:    Message           = Message()
-        self._loop:   AbstractEventLoop = get_event_loop()
-        self._prefix = prefix
+        self.id:    str | None        = None
+        self._db:   DB                = DB()
+        self._msg:  Message           = Message()
+        self._loop: AbstractEventLoop = new_event_loop()
+        self.prefix = prefix
 
         self.only_chats   = only_chats
         self.ignore_chats = ignore_chats
 
-        self.commands:    dict[str, dict[str, list[str] | Coro_return_None | str]] = {}
-        self.events:      dict[str, list[Coro_return_None]] = {
-                                                        'ready':      [],
-                                                        'close':      [],
-                                                        'message':    [],
-                                                        'join_chat':  [],
-                                                        'leave_chat': [],
-                                                        'image':      []
-                                                    }
+        self.commands: dict[str, dict[str, list[str], Coro_return_None, str]] = {}
+        self.events:   dict[str, list[Coro_return_None]] = {
+                                                    'ready':      [],
+                                                    'close':      [],
+                                                    'message':    [],
+                                                    'join_chat':  [],
+                                                    'leave_chat': [],
+                                                    'image':      []
+                                                }
 
     def add(
         self,
         help:    str       = 'No help',
         aliases: list[str] = []
-    ) -> Callable[[Coro_return_Message], None]:
+    ) -> Callable[[Coro_return_chat_message], None]:
 
-        def foo(f: Coro_return_Message) -> None:
+        def foo(f: Coro_return_chat_message) -> None:
             if not iscoroutinefunction(f):
                 raise CommandIsntAsync('Command must be async: "async def ..."')
             self.commands[f.__name__] = {
@@ -100,8 +119,8 @@ class Bot:
                                 }
         return foo
 
-    def on(self) -> Callable[[Coro_return_Message], None]:
-        def foo(f: Coro_return_Message) -> None:
+    def on(self) -> Callable[[Coro_return_chat_message], None]:
+        def foo(f: Coro_return_chat_message) -> None:
             if f.__name__ not in self.events:
                 raise InvalidEvent(f.__name__)
 
@@ -112,6 +131,18 @@ class Bot:
         return foo
 
     async def check_update(self) -> NoReturn | None:
+        def try_update() -> NoReturn | None:
+            if self._db.deps_need_update():
+                cmd = run('pip install -U amsync', capture_output=True, text=True)
+            else:
+                cmd = run('pip install -U amsync --no-deps', capture_output=True, text=True)
+
+            if cmd.returncode:
+                clear()
+                print(f'Error updating from version {Style.BRIGHT}{version}{Style.NORMAL} to {Fore.CYAN}{new}{Fore.WHITE}\n\n')
+                print(cmd.stdout or cmd.stderr)
+                sys.exit(1)
+
         if (
             'DYNO' not in environ   # not in heroku
             and self._db.lib_need_update()
@@ -120,21 +151,22 @@ class Bot:
                 async with s.get('https://pypi.org/pypi/Amsync/json') as res:
                     new = (await res.json(loads=loads))['info']['version']
                     if new != version:
-                        print(f'There is a new version: {Fore.CYAN}{new}{Fore.WHITE}\nDo you want to update it? (Y/n) ', end='')
+                        print(f'There is a new version: {Fore.CYAN}{new}{Fore.WHITE}')
+                        print(f'Actual version: {Style.BRIGHT}{version}{Style.NORMAL}\n')
+                        print(f'Do you want to update it? (Y/n) ', end='')
                         if input().lower() == 'y':
-                            obj.clear()
+                            clear()
                             print('Updating')
-                            if self._db.deps_need_update():
-                                run('pip install -U --force-reinstall --no-cache amsync', capture_output=True, text=True)
-                            run('pip uninstall -y amsync && pip install amsync', capture_output=True, text=True)
-                            obj.clear()
+                            try_update()
+                            clear()
                             print('Restarting...\n')
                             execl(sys.executable, Path(__file__).absolute(), *sys.argv)
-                        obj.clear()
+                        clear()
 
     def run(self) -> None:
         self._loop.run_until_complete(self.check_update())
         self._ws: Ws = Ws(
+            loop         = self._loop,
             email        = self._email,
             password     = self._password,
             only_chats   = self.only_chats,
@@ -165,7 +197,7 @@ class Bot:
         type_: int     | None = 0,
         embed: 'Embed' | None = None, # type: ignore
         com:   str     | None = None,
-        chat:  str     | None = None,
+        chat:  str     | None = None
     ) -> Awaitable[list[dict[str, Any]]]:
 
         return await self._msg.send(
@@ -177,7 +209,12 @@ class Bot:
             chat  = chat
         )
 
-    async def wait_for(self, check=lambda _: True, timeout=None):
+    async def wait_for(
+        self,
+        check: Callable[[Message.from_ws], bool] = lambda _: True,
+        timeout: int | None = None
+    ) -> Awaitable[Future, int | None] | None:
+
         future = self._loop.create_future()
         self._ws.futures.append(future)
 
@@ -191,15 +228,24 @@ class Bot:
             # Delete the future canceled by asyncio.wait_for
             del self._ws.futures[self._ws.futures.index(future)]
 
+    def _is_alias(self, name):
+        for command_name, args in self.commands.items():
+            if name in args['aliases']:
+                return command_name
+
     async def _call(self, m: Message) -> None:
-        if (
-            m.text
-            and m.text.startswith(self._prefix)
-            and (
-                (command_name := m.text.split()[0][len(self._prefix):])
-                in self.commands
-            )
-        ):
-            # Remove command name from text
-            m.text = ' '.join(m.text.split()[1:])
-            self._loop.create_task(self.commands[command_name]['def'](m))
+        if m.text and m.text.startswith(self.prefix):
+            splited      = m.text.split()
+            name         = splited[0][len(self.prefix):]
+            command_name = self._is_alias(name) or name
+
+            if command_name in self.commands:
+                if (
+                    len(splited) > 1 
+                    and splited[1] in (f'{self.prefix}h', f'{self.prefix}help')
+                ):
+                    await self.send(self.commands[command_name]['help'])
+                else:
+                    # Remove command name from text
+                    m.text = ' '.join(splited[1:])
+                    self._loop.create_task(self.commands[command_name]['def'](m))
