@@ -1,26 +1,30 @@
 from __future__ import annotations
+from re import M
 
-from typing import Any, Callable, NoReturn, Dict
+from uuid import uuid4
+from typing import Any, Awaitable, Callable, Literal, NoReturn, Dict
 from pathlib import Path
 from asyncio import gather
 from platform import system
+from datetime import datetime
 from subprocess import run
 from contextlib import suppress
 from unicodedata import normalize
 
+from ujson import dumps, loads, dump, load
+from aiohttp import request, ClientResponse
 from filetype import guess_mime
-from ujson import dumps, loads, dump
-from aiohttp import request
 from pybase64 import b64encode
 
 from .enum import MediaType
 from .exceptions import (
     SmallReasonForBan,
-    InvalidFileType
+    InvalidReturn,
+    AminoSays
 )
 
-
-headers: dict[str, str] = {'NDCDEVICEID': '0146BD6CF162E40F7449AFF316BA524DDDE1A1C4E6D99A553F3472EC3F4CB2F6A9ED05E5100492DC76'}
+ignore_codes = []
+headers: dict[str, str] = {'NDCDEVICEID': '0184a516841ba77a5b4648de2cd0dfcb30ea46dbb4e911a6ed122578b14c7b662c59f9a2c83101f5a1'}
 actual_com:  str | None = None
 actual_chat: str | None = None
 bot_id:      str | None = None
@@ -28,23 +32,29 @@ API = 'https://service.narvii.com/api/v1/'
 
 Req_json = Dict[str, Any]
 
-
+in_win = system() == 'Windows'
 def clear() -> None:
-    run('cls' if system() == 'Windows' else 'clear', shell=True)
+    run('cls' if in_win else 'clear', shell=True)
 
-def exist(
+def words(s):
+    return s.strip().count(' ')+1
+
+def get_value(
     d:       Req_json,
-    k:       str,
+    *k:      str,
     convert: Any | None = None
 ) -> Any | None:
 
+    tmp = None
     with suppress(KeyError):
-        if convert:
-            tmp = convert(d[k])
-            return tmp if tmp else None
-        return d[k]
+        for i in k:
+            tmp = d[i]
 
-def fix_ascii(s) -> str:
+        if convert:
+            return convert(tmp)
+        return tmp
+
+def fix_ascii(s: str) -> str:
     return normalize('NFKD', s).encode('ASCII', 'ignore').decode().strip()
 
 
@@ -55,16 +65,38 @@ def on_limit(
     return limit and len(obj) >= limit
 
 
-async def upload_media(file):
+async def upload_media(file: str) -> str:
     return (
         await _req(
             'post',
             '/g/s/media/upload',
-            data=await File.get(file),
+            data=await FILE.get(file),
             need_dumps=False,
         )
     )['mediaValue']
 
+async def upload_chat_bg(file: str) -> str:
+    return (
+        await _req(
+            'post',
+            'g/s/media/upload/target/chat-background',
+            data=await FILE.get(file),
+            need_dumps=False,
+        )
+    )['mediaValue']
+
+async def upload_chat_icon(file: str) -> str:
+    return (
+        await _req(
+            'post',
+            'g/s/media/upload/target/chat-cover',
+            data=await FILE.get(file),
+            need_dumps=False,
+        )
+    )['mediaValue']
+
+
+
 async def _req(
     method:     str,
     url:        str,
@@ -72,25 +104,25 @@ async def _req(
     data:       dict[str, Any] | None = None,
     need_dumps: bool                  = True,
     return_:    str                   = 'json',
-) -> Req_json | str | bytes:
+) -> Awaitable[Req_json | str | bytes] | ClientResponse:
 
-    async def get_return(res):
+    async with request(
+        method           = method,
+        url              = API + url,
+        data             = dumps(data) if need_dumps else data,
+        headers          = headers,
+    ) as res:
+        if res.status < 400 and (j := await res.json(loads=loads))['api:statuscode']:
+            raise AminoSays(f"{j['api:message']}. Code: {j['api:statuscode']}")
         if return_ == 'json':
             return await res.json(loads=loads)
         if return_ == 'text':
             return await res.text()
         if return_ == 'file':
             return await res.read()
-        raise Exception(f'Invalid return "{return_}"')
-
-    async with request(
-        method,
-        API + url,
-        data=dumps(data) if need_dumps else data,
-        headers=headers,
-        raise_for_status=True,
-    ) as _req:
-        return await get_return(_req)
+        if return_ == 'aiohttp':
+            return res
+        raise InvalidReturn(return_)
 
 
 class Message:
@@ -100,7 +132,6 @@ class Message:
         'com',
         'extensions',
         'file_link',
-        'has_mention',
         'icon',
         'id',
         'media_type',
@@ -108,70 +139,52 @@ class Message:
         'nickname',
         'text',
         'type',
-        'uid',
+        'uid'
     )
 
     def from_ws(self, j: Req_json) -> Message:
         _cm: dict[str, Any] = j['chatMessage']
 
         global actual_chat, actual_com
-        self.chat: str | None = exist(_cm, 'threadId', str)
-        self.com:  str | None = exist(j, 'ndcId', str)
+        self.chat: str | None = get_value(_cm, 'threadId', convert=str)
+        self.com:  str | None = get_value(j, 'ndcId', convert=str)
 
         actual_chat = self.chat
-        actual_com = self.com
+        actual_com  = self.com
 
-        self.extensions:  dict[str, Any] | None = exist(_cm, 'extensions')
-        self.file_link:   str | None            = exist(_cm, 'mediaValue')
-        self.has_mention: bool                  = (
-                                                True
-                                                if exist(self.extensions, 'mentionedArray')
-                                                else False
-                                            )
+        self.extensions:  dict[str, Any] | None = get_value(_cm, 'extensions')
+        self.file_link:   str | None            = get_value(_cm, 'mediaValue')
 
-        self.icon:            str | None       = exist(_cm['author'], 'icon') if 'author' in _cm else None
-        self.id:              str | None       = exist(_cm, 'messageId')
-        self.media_type:      str | None       = exist(_cm, 'mediaType')
-        self.mentioned_users: list[str] | None = (
-                                                [u['uid']
-                                                for u in self.extensions['mentionedArray']]
-                                                if self.has_mention
-                                                else None
-                                            )
+        self.icon:            str | None       = get_value(_cm, 'author', 'icon')
+        self.id:              str | None       = get_value(_cm, 'messageId')
+        self.media_type:      str | None       = get_value(_cm, 'mediaType')
+        self.mentioned_users: list[str] | None = [u.uid for u in get_value(self.extensions, 'mentionedArray') or []]
 
-        self.nickname: str | None = exist(_cm['author'], 'nickname') if 'author' in _cm else None
-        self.text:     str | None = exist(_cm, 'content')
-        self.type:     str | None = exist(_cm, 'type')
-        self.uid:      str | None = exist(_cm, 'uid')
+        self.nickname: str | None = get_value(_cm, 'author', 'nickname')
+        self.text:     str | None = get_value(_cm, 'content')
+        self.type:     str | None = get_value(_cm, 'type')
+        self.uid:      str | None = get_value(_cm, 'uid')
 
         return self
 
     def from_chat(self, j: Req_json) -> Message:
-        self.author:      User | None           = User(exist(j, 'author'))
-        self.chat:        str  | None           = exist(j, 'threadId')
-        self.extensions:  dict[str, Any] | None = exist(j, 'extensions')
-        self.has_mention: bool                  = (
-                                                True
-                                                if exist(self.extensions, 'mentionedArray')
-                                                else False
-                                            )
-        self.id:              str | None        = exist(j, 'messageId')
-        self.mentioned_users: list[str] | None  = (
-                                                [u['uid']
-                                                for u in j['extensions']['mentionedArray']]
-                                                if self.has_mention
-                                                else None
-                                            )
-        self.text: str | None = exist(j, 'content')
-        self.type: str | None = exist(j, 'type')
+        self.author:      User | None           = User(get_value(j, 'author'))
+        self.chat:        str  | None           = get_value(j, 'threadId')
+        self.extensions:  dict[str, Any] | None = get_value(j, 'extensions')
+
+        self.id:              str | None        = get_value(j, 'messageId')
+        self.mentioned_users: list[str] | None  = [u['uid'] for u in get_value(self.extensions, 'mentionedArray') or []]
+        self.text: str | None = get_value(j, 'content')
+        self.type: str | None = get_value(j, 'type')
+
         return self
 
     class _CreateData:
-        async def msg(type_, msgs):
-            return [{'type': type_, 'content': i} for i in msgs]
+        async def msg(type, msgs):
+            return [{'type': type, 'content': i} for i in msgs]
 
         async def file(files):
-            return [await File().process(i) for i in files]
+            return [await FILE.process(i) for i in files]
 
         async def embed(embed: Embed):
             if embed.image:
@@ -198,8 +211,8 @@ class Message:
         chat:  str | None             = None,
     ) -> list[Req_json]:
 
-        com = actual_com or com
-        chat = actual_chat or chat
+        com = com or actual_com
+        chat = chat or actual_chat
         files = [files] if files and not isinstance(files, (tuple, list)) else files
 
         if msgs:
@@ -220,14 +233,27 @@ class Message:
 
 
 class Embed:
-    __slots__ = ('msg_text', 'title', 'text', 'link', 'image')
+    __slots__ = (
+        'msg_text',
+        'title',
+        'text',
+        'link',
+        'image'
+    )
 
-    def __init__(self, msg_text, title, text, link, image=None):
+    def __init__(
+        self,
+        msg_text: str,
+        title:    str,
+        text:     str,
+        link:     str,
+        image:    str | bytes = None
+    ):
         self.msg_text = msg_text
-        self.title = title
-        self.text = text
-        self.link = link
-        self.image = image
+        self.title    = title
+        self.text     = text
+        self.link     = link
+        self.image    = image
 
 
 class User:
@@ -252,42 +278,48 @@ class User:
 
     def __init__(self, j: Req_json | None = None):
         if j:
-            self.bio             = exist(j, 'content')
-            self.blogs_count     = exist(j, 'blogsCount')
-            self.com             = exist(j, 'ndcId', str)
-            self.comments_count  = exist(j, 'commentsCount')
-            self.created_time    = exist(j, 'createdTime')
-            self.followers_count = exist(j, 'membersCount')
-            self.following_count = exist(j, 'joinedCount')
-            self.im_following    = j['followingStatus'] == 1 if 'followingStatus' in j else None
-            self.level           = exist(j, 'level')
-            self.nickname        = exist(j, 'nickname')
-            self.posts_count     = exist(j, 'postsCount')
-            self.id              = exist(j, 'uid')
-            self.reputation      = exist(j, 'reputation')
+            self.bio             = get_value(j, 'content')
+            self.blogs_count     = get_value(j, 'blogsCount')
+            self.com             = get_value(j, 'ndcId', convert=str)
+            self.comments_count  = get_value(j, 'commentsCount')
+            self.created_time    = get_value(j, 'createdTime')
+            self.followers_count = get_value(j, 'membersCount')
+            self.following_count = get_value(j, 'joinedCount')
+            self.im_following    = get_value(j, 'followingStatus') == 1
+            self.level           = get_value(j, 'level')
+            self.nickname        = get_value(j, 'nickname')
+            self.posts_count     = get_value(j, 'postsCount')
+            self.id              = get_value(j, 'uid')
+            self.reputation      = get_value(j, 'reputation')
             self.role            = {0: 'member', 101: 'curator', 100: 'leader', 102: 'leader-agent'}[j['role']]
-            self.visitors_count  = exist(j, 'visitoresCount')
+            self.visitors_count  = get_value(j, 'visitoresCount')
 
     @classmethod
     async def search(
-        cls, uids: str | list[str], com: str | None = None
-    ) -> list[User]:
+        cls,
+        uids: str | list[str],
+        com:  str | None = None
+    ) -> Awaitable[list[User]]:
 
-        com = actual_com or com
+        com = com or actual_com
         uids = [uids] if not isinstance(uids, (list, tuple)) else uids
 
         async def foo(uid: str) -> User:
             return cls(
-                (await _req('get', f'x{com}/s/user-profile/{uid}'))[
-                    'userProfile'
-                ]
+                (await _req('get', f'x{com}/s/user-profile/{uid}'))['userProfile']
             )
 
         return await gather(*[foo(uid) for uid in uids])
 
-    async def ban(self, uid, *, reason, com=None):
-        com = actual_com or com
-        if len(reason.split()) < 3:
+    async def ban(
+        self,
+        uid:    str,
+        *,
+        reason: str,
+        com:    str | None = None
+    ):
+        com = com or actual_com
+        if words(reason) < 3:
             raise SmallReasonForBan('Put a reason with at least three words')
 
         return await _req(
@@ -296,7 +328,13 @@ class User:
             data={'reasonType': 200, 'note': {'content': reason}},
         )
 
-    async def unban(self, uid, *, reason='', com=None):
+    async def unban(
+        self,
+        uid:    str,
+        *,
+        reason: str        = '',
+        com:    str | None = None
+    ):
         com = com or actual_com
 
         return await _req(
@@ -308,7 +346,7 @@ class User:
 
 class File:
     @staticmethod
-    def type_(file):
+    def type(file: str | bytes) -> Literal[MediaType.LINK, MediaType.BYTES, MediaType.PATH]:
         if isinstance(file, str) and file.startswith('http'):
             return MediaType.LINK
 
@@ -318,8 +356,8 @@ class File:
         return MediaType.PATH
 
     @staticmethod
-    async def get(file: str) -> bytes:
-        type = File.type_(file)
+    async def get(file: str | bytes) -> bytes:
+        type = File.type(file)
 
         if type == MediaType.LINK:
             async with request('get', file) as res:
@@ -337,43 +375,38 @@ class File:
 
     async def process(self, file: str | bytes) -> dict[str, Any] | NoReturn:
         if (
-            self.type_(file) not in (MediaType.LINK, MediaType.BYTES)
+            self.type(file) not in (MediaType.LINK, MediaType.BYTES)
             and not Path(file).exists()
         ):
             raise FileNotFoundError(file)
 
         b = await self.get(file)
+        b64 = self.b64(b)
+        type = (guess_mime(b) or 'audio/mp3').split('/')
 
-        # Only first 261 bytes representing the max file header is required
-        #
-        # https://github.com/h2non/filetype.py#features
-        type_ = guess_mime(b[:261]).split('/')
-
-        if type_[1] == 'gif':
+        if type[-1] == 'gif':
                 return {
                 'mediaType': 100,
-                'mediaUploadValue': self.b64(b),
+                'mediaUploadValue': b64,
                 'mediaUploadValueContentType': 'image/gif',
                 'mediaUhqEnabled': True,
             }
 
-        if type_[0] == 'image':
+        if type[0] == 'image':
             return {
                 'mediaType': 100,
-                'mediaUploadValue': self.b64(b),
+                'mediaUploadValue': b64,
                 'mediaUhqEnabled': True,
             }
 
-        if type_[0] == 'audio':
+        if type[-1] == 'mp3':
             return {
                 'type': 2,
                 'mediaType': 110,
-                'mediaUploadValue': self.b64(b),
+                'mediaUploadValue': b64,
                 'mediaUhqEnabled': True,
             }
 
-        raise InvalidFileType(file if type_ != MediaType.BYTES else '')
-
 
 class Chat:
     __slots__ = (
@@ -381,7 +414,8 @@ class Chat:
         'bg',
         'can_send_coins',
         'co_hosts',
-        'creator',
+        'created_in',
+        'adm',
         'extensions',
         'icon',
         'id',
@@ -389,39 +423,42 @@ class Chat:
         'is_private',
         'members_can_invite',
         'name',
-        'text',
-        'title',
-        'view_only'
+        'only_fans',
+        'only_view',
+        'text'
     )
 
     def __init__(self, j=None):
         if j:
-            t = j['thread']
-            self.extensions         = exist(t, 'extensions')
+            t = get_value(j, 'thread') or j
+            self.extensions         = get_value(t, 'extensions')
 
-            self.announcement       = exist(self.extensions, 'announcement') or ''
-            self.bg                 = exist(self.extensions, 'bm')[1] if 'bm' in self.extensions else None
-            self.can_send_coins     = exist(t['tipInfo'], 'tippable') if 'tipInfo' in t else None
-            self.co_hosts           = exist(self.extensions, 'coHost') or []
-            self.creator            = exist(t['author'], 'uid')
-            self.icon               = exist(t, 'icon')
-            self.id                 = exist(t, 'threadId')
-            self.is_pinned          = exist(t, 'isPinned')
-            self.is_private         = exist(t, 'membersQuota') == 2
-            self.members_can_invite = exist(t, 'membersCanInvite')
-            self.name               = exist(t, 'title')
-            self.text               = '' if exist(t, 'content') in ('None', None) else exist(t, 'content')
-            self.view_only          = exist(self.extensions, 'viewOnly') or False
+            self.announcement       = get_value(self.extensions, 'announcement') or ''
+            self.bg                 = get_value(self.extensions, 'bm')[1] if 'bm' in self.extensions else None
+            self.can_send_coins     = get_value(t, 'tipInfo', 'tippable')
+            self.co_hosts           = get_value(self.extensions, 'coHost') or []
+            self.created_in         = int(datetime.fromisoformat(t['createdTime'][:-1]).timestamp()) if t['createdTime'] else None
+            self.adm                = get_value(t, 'author', 'uid')
+            self.icon               = get_value(t, 'icon')
+            self.id                 = get_value(t, 'threadId')
+            self.is_pinned          = get_value(t, 'isPinned')
+            self.is_private         = get_value(t, 'membersQuota') == 2
+            self.members_can_invite = get_value(t, 'membersCanInvite') or True
+            self.name               = get_value(t, 'title')
+            self.only_fans          = get_value(self.extensions, 'fansOnly')
+            self.only_view          = get_value(self.extensions, 'viewOnly') or False
+            self.text               = get_value(t, 'content') or ''
 
     @classmethod
-    async def search(cls, com=None, chat=None):
-        com = actual_com or com
-        chat = actual_chat or chat
+    async def search(
+        cls,
+        com:  str | None = None,
+        chat: str | None = None
+    ):
+        com = com or actual_com
+        chat = chat or actual_chat
 
-        return cls(await _req(
-            'get',
-            f'x{com}/s/chat/thread/{chat}'
-        ))
+        return cls(await _req('get',f'x{com}/s/chat/thread/{chat}'))
 
     async def messages(
         self,
@@ -433,8 +470,8 @@ class Chat:
         end:   int | None = None,
     ) -> list[Message.from_chat]:
 
-        com = actual_com or com
-        chat = actual_chat or chat
+        com = com or actual_com
+        chat = chat or actual_chat
         messages = []
 
         res = await _req(
@@ -475,14 +512,13 @@ class Chat:
         end:   int | None = None,
     ) -> list[Req_json]:
 
-        com = actual_com or com
-        chat = actual_chat or chat
+        com = com or actual_com
+        chat = chat or actual_chat
         msgs = (
             ([msgs] if not isinstance(msgs, (tuple, list)) else msgs)
             if msgs
             else [
-                msg.id
-                for msg in await self.messages(
+                msg.id for msg in await self.messages(
                     check=check, com=com, chat=chat, start=start, end=end
                 )
             ]
@@ -504,9 +540,10 @@ class Chat:
         chat:  str | None = None,
         start: int | None = None,
         end:   int | None = None,
-    ):
-        com = actual_com or com
-        chat = actual_chat or chat
+    ) -> list[User]:
+
+        com = com or actual_com
+        chat = chat or actual_chat
 
         async def foo(i):
             res = await _req(
@@ -514,29 +551,33 @@ class Chat:
                 f'x{com}/s/chat/thread/{chat}/member?start={i}&size=100&type=default&cv=1.2',
             )
             return [
-                i
-                for i in [
+                i for i in [
                     User(i) for i in res['memberList'] if res['memberList']
                 ]
                 if check(i)
             ]
 
-        members_count = (await _req('get', f'x{com}/s/chat/thread/{chat}'))[
-            'thread'
-        ]['membersCount']
+        members_count = (await _req(
+            'get', f'x{com}/s/chat/thread/{chat}'
+        ))['thread']['membersCount']
+
         MAX_MEMBERS_COUNT_IN_CHAT = 1000
         return (
             await gather(
                 *[
-                    foo(i)
-                    for i in range(0, MAX_MEMBERS_COUNT_IN_CHAT, 100)
+                    foo(i) for i in range(0, MAX_MEMBERS_COUNT_IN_CHAT, 100)
                     if i <= members_count
                 ]
             )
         )[0][start:end]
 
-    async def join(self, chat: str, com: str | None = None):
-        com = actual_com or com
+    async def join(
+        self,
+        chat: str, 
+        com: str | None = None
+    ) -> Req_json:
+
+        com = com or actual_com
         chat = [chat] if not isinstance(chat, (list, tuple)) else chat
 
         async def foo(i):
@@ -546,8 +587,13 @@ class Chat:
 
         return await gather(*[foo(i) for i in chat])
 
-    async def leave(self, chat: str, com: str | None = None):
-        com = actual_com or com
+    async def leave(
+        self,
+        chat: str,
+        com: str | None = None
+    ) -> Req_json:
+
+        com = com or actual_com
         chat = [chat] if not isinstance(chat, (list, tuple)) else chat
 
         async def foo(i):
@@ -557,7 +603,136 @@ class Chat:
 
         return await gather(*[foo(i) for i in chat])
 
-    async def save(self, filename=None):
+    async def create(
+        self,
+        name:           str,
+        text:           str | None  = None,
+        bg:             str | bytes = None,
+        icon:           str | bytes = None,
+        only_fans:      bool        = False,
+        invite_members: list[str]   = [],
+        com:            str | None  = None
+    ) -> Req_json:
+
+        com = com or actual_com
+
+        img = [100, await upload_chat_bg(bg), None] if bg else bg
+        data = {
+            'backgroundMedia': img,
+            'extensions': {
+                'bm': img,
+                'fansOnly': only_fans
+            },
+            'title': name,
+            'content': text,
+            'icon': await upload_chat_icon(icon) if icon else icon,
+            'inviteeUids': invite_members,
+
+            # need this to work
+            'type': 2,
+            'eventSource': 'GlobalComposeMenu'
+        }
+
+        return await _req('post', f'x{com}/s/chat/thread', data=data)
+
+    async def delete(
+        self,
+        com:  str | None = None,
+        chat: str | None = None
+    ) -> bool:
+
+        com = com or actual_com
+        chat = chat or actual_chat
+
+        return (await _req('delete', f'x{com}/s/chat/thread/{chat}', return_='aiohttp')).ok
+
+    async def config(
+        self,
+        name:               str | None         = None,
+        text:               str | None         = None,
+        bg:                 str | bytes | None = None,
+        pin:                bool | None        = None,
+        announcement:       str | None         = None,
+        only_view:          bool | None        = None,
+        members_can_invite: bool | None        = None,
+        can_send_coins:     bool | None        = None,
+        change_adm_to:      str | None         = None,
+        com:                str | None         = None,
+        chat:               str | None         = None
+    ) -> None:
+
+        com = com or actual_com
+        chat = chat or actual_chat
+
+        info = await self.search(chat=chat)
+        if name or text:
+            data = {
+                'extensions': {
+                    'bm': [100, await upload_chat_bg(bg), None] if bg else bg,
+                    'fansOnly': info.only_fans
+                },
+                'title': name or info.name,
+                'content': text or info.text,
+                'icon': await upload_chat_icon(info.icon) if info.icon else info.icon,
+
+                # need this to work
+                'type': 2,
+                'eventSource': 'GlobalComposeMenu'
+            }
+            await _req('post', f'x{com}/s/chat/thread/{chat}', data=data)
+
+
+        if bg:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/member/{bot_id}/background', data=await FILE.process(bg))
+        elif bg == False:
+            await _req('delete', f'x{com}/s/chat/thread/{chat}/member/{bot_id}/background')
+
+        if pin:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/pin')
+        elif pin == False:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/unpin')
+
+        if announcement:
+            await _req('post', f'x{com}/s/chat/thread/{chat}', data={'announcement': announcement, 'pinAnnouncement': True})
+        elif announcement == False:
+            await _req('post', f'x{com}/s/chat/thread/{chat}', data={'pinAnnouncement': False})
+
+        if only_view:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/view-only/enable')
+        elif only_view == False:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/view-only/disable')
+
+        if members_can_invite:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/members-can-invite/enable')
+        elif members_can_invite == False:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/members-can-invite/disable')
+
+        if can_send_coins:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/tipping-perm-status/enable')
+        elif can_send_coins == False:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/tipping-perm-status/disable')
+
+        if change_adm_to:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/transfer-organizer', data={'uidList': [change_adm_to]})
+
+    async def change_co_hosts(
+        self,
+        add:    list[str] | str | None = None,
+        remove: str | None = None,
+        com:    str | None = None,
+        chat:   str | None = None
+    ) -> None:
+
+        com = com or actual_com
+        chat = chat or actual_chat
+        add = [add] if add and not isinstance(add, (list, tuple)) else add
+
+        if add:
+            await _req('post', f'x{com}/s/chat/thread/{chat}/co-host', data={'uidList': add})
+        elif remove:
+            await _req('delete', f'x{com}/s/chat/thread/{chat}/co-host/{remove}')
+
+    async def save(self, filename: str | None = None) -> None:
         chat = await Chat.search()
         info = {
             'name': chat.name,
@@ -567,12 +742,14 @@ class Chat:
             'bg': chat.bg,
             'icon': chat.icon,
 
-            'creator': chat.creator,
-            'co-hosts': chat.co_hosts,
+            'adm': chat.adm,
+            'co_hosts': chat.co_hosts,
 
-            'can-send-coins': chat.can_send_coins,
-            'is-pinned': chat.is_pinned,
-            'only-view': chat.view_only,
+            'members_can_invite': chat.members_can_invite,
+            'can_send_coins': chat.can_send_coins,
+            'is_pinned': chat.is_pinned,
+            'only_view': chat.only_view,
+            'only_fans': chat.only_fans,
 
             'members': [i.id for i in await self.members()]
         }
@@ -581,12 +758,47 @@ class Chat:
         while Path(f'{n}.json').exists():
             n += 1
         with open(filename or f'{n}.json', 'w') as f:
-            dump(info, f, indent=4, ensure_ascii=False, escape_forward_slashes=False)
+            dump(info, f, indent=4, escape_forward_slashes=False)
 
+    async def load(self, filename: str) -> None:
+        with open(filename, 'r') as f:
+            f = load(f)
+
+        tmp_chat_name = str(uuid4())
+        await self.create(
+            name = tmp_chat_name,
+            text = f['text'],
+            bg = f['bg'],
+            icon = f['icon'],
+            only_fans= f['only_fans'],
+            invite_members=f['members']
+        )
+
+        chats = list((await Community.chats()).values())[0]
+        names = [i.name for i in chats]
+        ids = [i.id for i in chats]
+
+        await self.config(
+            name               = f['name'],
+            pin                = f['is_pinned'],
+            announcement       = f['announcement'],
+            only_view          = f['only_view'],
+            members_can_invite = f['members_can_invite'],
+            can_send_coins     = f['can_send_coins'],
+            change_adm_to      = f['adm'] if f['adm'] != bot_id else None,
+            chat               = ids[names.index(tmp_chat_name)]
+        )
+
+        await self.change_co_hosts(f['co_hosts'])
 
 class Community:
     @staticmethod
-    async def chats(com=None, need_print=False, ignore_ascii=False):
+    async def chats(
+        need_print:   bool       = False,
+        ignore_ascii: bool       = False,
+        com:          str | None = None
+    ) -> dict[str, list[Chat]]:
+
         if not actual_com and not com:
             raise Exception('Enter a com or send a message in a chat')
 
@@ -624,7 +836,11 @@ class Community:
 
 class My:
     @staticmethod
-    async def chats(need_print=True, ignore_ascii=False):
+    async def chats(
+        need_print:   bool = True,
+        ignore_ascii: bool = False
+    ) -> dict[str, list[str, list[str]]]:
+
         res = await _req('get', 'g/s/community/joined?v=1&start=0&size=50')
         coms = {str(i['ndcId']): [i['name'], []] for i in res['communityList']}
 
@@ -667,7 +883,11 @@ class My:
         return coms
 
     @staticmethod
-    async def communities(need_print=True, ignore_ascii=False):
+    async def communities(
+        need_print:   bool = True,
+        ignore_ascii: bool = False
+    ) -> dict[str, str]:
+
         res = await _req('get', f'g/s/community/joined?v=1&start=0&size=50')
         coms = {
             i['name']
@@ -683,3 +903,11 @@ class My:
                 print(f'{i} {" "*a} -> {e}')
 
         return coms
+
+
+# # # # # # #
+#   Cache   #
+# # # # # # #
+
+FILE    = File()
+MESSAGE = Message()
