@@ -1,7 +1,8 @@
 from __future__ import annotations
 
-from typing import Any, Callable, AsyncIterator, Coroutine
-from asyncio import get_event_loop, AbstractEventLoop, Future
+from re import search
+from typing import AsyncIterator
+from asyncio import new_event_loop, AbstractEventLoop, Future
 from contextlib import suppress
 
 from ujson import dumps, loads
@@ -13,21 +14,31 @@ from .db import DB
 from .obj import Message, _req
 from .enum import WsStatus
 
-Coro_return_None = Callable[[], Coroutine[None, None, None]]
-Coro_return_Message = Callable[[], Coroutine[Message, None, None]]
-Coro_return_Any = Callable[[], Coroutine[Any, None, None]]
-
 
 class Ws:
+    __slots__ = (
+        '_db',
+        '_deviceid',
+        '_email',
+        '_events',
+        '_ignore_chats',
+        '_loop',
+        '_only_chats',
+        '_password',
+        '_status',
+        'futures'
+    )
+
     def __init__(
         self,
-        email:        str,
-        password:     str,
-        only_chats:   dict[str, list[str]] = {},
-        ignore_chats: dict[str, list[str]] = {}
+        loop:          AbstractEventLoop, 
+        email:        'Bot.email',             # type: ignore
+        password:     'Bot.password',          # type: ignore
+        only_chats:   'Bot.only_chats'   = {}, # type: ignore
+        ignore_chats: 'Bot.ignore_chats' = {}  # type: ignore
     ):
         self._deviceid: str               = obj.headers['NDCDEVICEID']
-        self._loop:     AbstractEventLoop = get_event_loop()
+        self._loop:     AbstractEventLoop = loop
         self._db:       DB                = DB()
         self.futures:   list[Future]      = []
 
@@ -35,6 +46,7 @@ class Ws:
         self._password     = password
         self._only_chats   = only_chats
         self._ignore_chats = ignore_chats
+        self._status       = WsStatus.OPEN
 
 
     async def _get_sid(self) -> str:
@@ -51,8 +63,8 @@ class Ws:
                 headers=obj.headers,
             )
             self._call_events('ready')
-
-            while True:
+            
+            while self._status == WsStatus.OPEN: # for tests
                 if ws.closed:
                     self._call_events('close')
                     yield WsStatus.CLOSED
@@ -64,11 +76,21 @@ class Ws:
                     if res['t'] == 1000:
                         yield Message().from_ws(res['o'])
 
-    def _call_events(self, event):
-        for i in self._events[event]:
-            self._loop.create_task(i())
+    def _call_events(
+        self,
+        name: str,
+        *m: list[Message.from_ws]
+    ):
+        for i in self._events[name]:
+            self._loop.create_task(i(*m))
 
-    def _can_call(self, msg: Message):
+    def _fix_sid(self, sid: str):
+        # Sometimes sid can be an invalid base64,
+        # causing a padding error in urlsafe_b64decode
+        # Add = until sid has the len of 192 or len % 4 == 0 solve the problem
+        return sid + '=' * (192 - len(sid))
+
+    def _can_call(self, msg: Message.from_ws):
         if not self._only_chats and not self._ignore_chats:
             return True
 
@@ -90,23 +112,21 @@ class Ws:
 
     async def run(
         self,
-        call:   Coro_return_None,
-        events: dict[str, list[Coro_return_Any]],
-        bot:    'Bot' # type: ignore
+        call:   'Bot._call',  # type: ignore
+        events: 'Bot.events', # type: ignore
+        bot:    'Bot'         # type: ignore
     ) -> None:
+    
         if not self._db.get_account(self._email):
-            self._db.add_account(self._email, await self._get_sid())
+            self._db.add_account(self._email, self._fix_sid(await self._get_sid()))
 
-        # Sometimes sid can be an invalid base64,
-        # causing a padding error in urlsafe_b64decode
-        # Add = until sid has the len of 192 solve the problem
-        tmp = self._db.get_account(self._email)
-        sid = tmp + '=' * (192 - len(tmp))
+        sid = self._db.get_account(self._email)
 
         obj.headers['NDCAUTH'] = f'sid={sid}'
-        id_ = loads(
-            urlsafe_b64decode(sid[:-28])[1:] + b'}'
-        )['2']
+        id_ = search(
+            r'\w{8}-\w{4}-\w{4}-\w{4}-\w{12}',
+            urlsafe_b64decode(sid).decode('cp437'),
+        ).group(0)
 
         bot.id = id_
         obj.bot_id = id_
@@ -122,11 +142,12 @@ class Ws:
 
         async for m in self._connect():
             if m == WsStatus.CLOSED:
+                # Reconnect
                 return await self.run(call, self._events, bot)
 
             if self._can_call(m):
                 with suppress(KeyError):
-                    self._call_events(events[f'{m.type}:{m.media_type}'])
+                    self._call_events(events[f'{m.type}:{m.media_type}'], m)
 
                 if self.futures:
                     for future in self.futures:
